/*
 * Copyright (c) 2007 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

#define nop()	__asm__ volatile ("nop" )

#include <usbconfig.h>
#include <cdc162.h>

char debug_buf[64];
uint8_t debug_buf_len;

int main(void)
{
    uint8_t ch;

#ifdef F_SCALER
    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = F_SCALER;
#endif

    // bit_set(REGCR, REGDIS); // disable internal regulator 

    //bit_set(DDRB, PB2);
    //bit_set(PORTB, PB2); // LED ON

    ch = 20;
    while (--ch) {
        _delay_ms(1);
    }
    usbcdc_init();

    sei();

    for(;;) {
        usbcdc_poll();
	if (debug_buf_len) {
		uint8_t i;
		for (i=0; i< debug_buf_len; i++) {
			__usbcdc_putc_nc(debug_buf[i]);
		}
		debug_buf_len = 0;
	}
	if (usbcdc_can_getc(1)) {
	    ch = usbcdc_getc();
	    usbcdc_putc(ch);
	}
    }
    return 0;
}

#if CDC162_EXCONT
static void debug_puts(char *str) {
	while (*str) {
		if (debug_buf_len < sizeof(debug_buf)) {
			debug_buf[debug_buf_len++] = *str;
		}
		str++;
	}
}

static void debug_put_hex(uint8_t dat) {
	static char *x = "012345678ABCDEF";
	char buf[3];
	buf[0] = x[(dat >> 4)& 0xf];
	buf[1] = x[dat & 0xf];
	buf[2] = 0;
	debug_puts(buf);
}

uint8_t control_mode;
struct acm_linemode linemode;

void callback_set_linemode(struct acm_linemode* lm) {
	linemode = *lm;
	// Baud = lm->baud;
}

void callback_set_control(uint8_t mode) {
	if (mode & ACM_CTRL_DTR) {
	}
	if (mode & ACM_CTRL_RTS) {
	}
	control_mode = mode;
}

void callback_send_break(int16_t ms) {
	debug_puts("\r\n*** BREAK 0x");
	debug_put_hex((uint8_t)(ms & 0xff));
	debug_puts("\r\n CTRL DTR:");
	if (control_mode & ACM_CTRL_DTR) {
		debug_puts("ON");
	} else {
		debug_puts("OFF");
	}
	debug_puts(" RTS:");
	if (control_mode & ACM_CTRL_RTS) {
		debug_puts("ON");
	} else {
		debug_puts("OFF");
	}
	debug_puts("\r\n LINE: STOP 0x");
	debug_put_hex(linemode.stopbit);
	debug_puts(" PAR 0x");
	debug_put_hex(linemode.parity);
	debug_puts(" DAT 0x");
	debug_put_hex(linemode.databit);
	debug_puts("\r\n");
}
#else
void callback_set_baud(long baud) {
	// Baud = baud;
}
#endif
