//********************************************************************
//	R8C Series programmer at AT90USB162
//
//	Copyright (c)2009	KAWAKAMI Yukio
//	2009.10.9	Version 0.1.1
//	2009.4.19	Version 0.1.0
//	Copyright (c)2010	Koji Suzuki	
//	2010.12.7	ported to angel_loader/serial
//********************************************************************
/*
 * Copyright (c) 2007 Koji Suzuki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include <usbconfig.h>
#include <cdc162.h>

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

#define nop()	__asm__ volatile ("nop" )
#define	wdr()	__asm__ volatile("wdr" )

long	Baud = 0;

uint8_t	pdi_tpi_mode;

static inline void set_pdi_tpi_mode(int8_t is_on) {
	if (is_on) {
    		bit_set(PORTD,PD5);	//
    		bit_set(DDRD,PD5);	// XCK = H

#if (F_CPU == 16000000UL)
		UBRR1 =  0;	// F_CPU / 16 = 0.5 MHz @ 8MHz
#else
		UBRR1 =  0;	// F_CPU / 16 = 0.5 MHz @ 8MHz
#endif
		bit_clear(UCSR1A , U2X1); // must clear in syncronus mode

		UCSR1C = (1<<UMSEL10) | (2<<UPM10) | (1<<USBS1)
			| (3<<UCSZ10) | (1<<UCPOL1);
	}
}

// USART Rx buffer
volatile uint8_t	Rbuff[240];
volatile uint8_t	Rremain = 0;
uint8_t			Rin = 0;
uint8_t			Rout = 0;

// USART Tx buffer
volatile uint8_t	Sbuff[160];
volatile uint8_t	Sremain = 0;
uint8_t			Sin = 0;
uint8_t			Sout = 0;

volatile uint8_t	led_tx_timer;
volatile uint8_t	led_rx_timer;

#include <serialconfig.h>

#if 0
#define MIN_LIGHTING_TIME 	10	// x 10ms
static inline void led_tx_on() {
	bit_set(PORTD, PD7)
}
static inline void led_tx_off() {
	bit_clear(PORTD, PD7);
}
static inline void led_rx_on() {
	bit_set(PORTD, PD6)
}
static inline void led_rx_off() {
	bit_clear(PORTD, PD6);
}

static inline void led_init() {
	bit_set(DDRD, PD6);
	bit_set(DDRD, PD7);
}
#endif

static inline void led_proc(int ch) {
	if ((ch >= 0) || Sremain) {
		if (led_tx_timer < MIN_LIGHTING_TIME)
			 led_tx_timer = MIN_LIGHTING_TIME;
	}
	if (Rremain) {
		if (led_rx_timer < MIN_LIGHTING_TIME)
			 led_rx_timer = MIN_LIGHTING_TIME;
	}
}

ISR(TIMER1_COMPA_vect) {
	if (led_tx_timer) {
		led_tx_on();
		--led_tx_timer;
	} else {
		led_tx_off();
	}
	if (led_rx_timer) {
		led_rx_on();
		--led_rx_timer;
	} else {
		led_rx_off();
	}
}

ISR(USART1_RX_vect) {
	uint8_t c;
	c = UDR1;
	if (!(UCSR1A & ((1<<FE1)|(1<<DOR1)|(1<<UPE1)))) { // no error
		Rbuff[Rin++] = c;
		if (Rin >= sizeof(Rbuff)){
			Rin = 0;
		}
		Rremain++;
	}
}

ISR(USART1_UDRE_vect) {
	if (Sremain) {
		UDR1 = Sbuff[Sout++];
		if (Sout >= sizeof(Sbuff))	Sout = 0;
		--Sremain;
	}
}

ISR(USART1_TX_vect){
	bit_clear(UCSR1B, TXEN1);	// TX disable
	bit_set(UCSR1B, RXEN1);		// enable recv
#ifdef DIR_BIT
	bit_clear(DIR_PORT, DIR_BIT);
#endif
}

void send_com(uint8_t data) {
	if (Sremain || bit_is_clear(UCSR1A, UDRE1)) {
		cli();
		Sbuff[Sin++] = data;
		++Sremain;
		sei();
		if (Sin >= sizeof(Sbuff))	Sin = 0;
	} else {
#ifdef DIR_BIT
		bit_set(DIR_PORT, DIR_BIT);
#endif
		bit_clear(UCSR1B, TXCIE1);
		// enable TX, RX 
		UCSR1B |= (1<<TXEN1)|(1<<RXEN1)|(1<<UDRIE1);
		UDR1 = data;
	}
}

static inline void set_baud(const long baud) {
	if (baud == 115200) {
		UBRR1H = 0;
#if (F_CPU == 16000000UL)
		UBRR1L = 16;	// 2.1%
#else
		UBRR1L = 8;	// -3.5%
#endif
		bit_set(UCSR1A, U2X1);	// 1/8
		bit_set(UCSR1C, USBS1);	// Tx:stop2 Rx: stop1
	} else if(baud == 57600) {
		UBRR1H = 0;
#if (F_CPU == 16000000UL)
		UBRR1L = 34;	// -0.8%
		bit_clear(UCSR1C, USBS1);	// stop1
#else
		UBRR1L = 16;	// 2.1%
		bit_set(UCSR1C, USBS1);	// Tx:stop2 Rx: stop1
#endif
		bit_set(UCSR1A, U2X1);	// 1/8
	} else {
		UBRR1H = ((F_CPU/16/baud - 1) >> 8) & 0xff;
		UBRR1L =  (F_CPU/16/baud - 1) & 0xff;
		bit_clear(UCSR1A , U2X1);	// 1/16
		bit_clear(UCSR1C , USBS1);	// stop1
	}
}

#if CDC162_EXCONT
void callback_set_linemode(struct acm_linemode* lm) {
	Baud = lm->baud;
}

void callback_set_control(uint8_t mode) {
	if (mode & ACM_CTRL_DTR) {
	}
	if (mode & ACM_CTRL_RTS) {
	}
	led_rx_timer = 100;
}

void callback_send_break(int16_t ms) {
	led_tx_timer = 100;
}
#else
void callback_set_baud(long baud) {
	Baud = baud;
}
#endif

int main(void) {
    uint8_t c;
    int	ch = -1;

    cli();
    wdt_reset();
    wdt_disable();

    // bit_set(REGCR, REGDIS); // disable internal regulator 

#ifdef F_SCALER
    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = F_SCALER;
#endif

    bit_clear(PORTD,PD2);
    bit_set(PORTD,PD2);   // pullup RX
    bit_clear(PORTD,PD3);
    bit_set(PORTD,PD3);	// pullup TX

    // 100Hz timer
    TCCR1A = 0;
    TCCR1B = (1<<WGM12)|(3<<CS10);  // CTC  1/64
    TCCR1C = 0;
    OCR1AH = ((F_CPU/64/100 - 1) >> 8) & 0xff;
    OCR1AL =  (F_CPU/64/100 - 1) & 0xff;
    TIMSK1 = (1<<OCIE1A);  

    c = 20;
    while (--c) {
        _delay_ms(1);
    }
    usbcdc_init();

    wdt_enable(WDTO_1S);

    UCSR1C = (1<<UCSZ11)|(1<<UCSZ10);	// 8bit/stop 1/parity non
    UCSR1D = 0;				// Disable CTS, RTS
    //Baud = 9600L;
    set_baud(9600L);

    //enable interrputs
    UCSR1B = (1<<RXEN1)|(1<<RXCIE1);

    sei();

    led_init();
    for (;;){

	wdt_reset();
	usbcdc_poll();

	if (Baud) {
		set_baud(Baud);
		Baud = 0;
	}
	if ((ch < 0)&&(usbcdc_can_getc(1))){
		ch = usbcdc_getc();
	}
	led_proc(ch);
	if (ch >= 0){
		if (Sremain < (sizeof(Sbuff) - 1)){
			send_com((uint8_t)ch);
			ch = -1;
		}
	}
	while (Rremain) {
		c = Rbuff[Rout++];
		if (Rout >= sizeof(Rbuff)){
			Rout = 0;
		}
		cli();
		--Rremain;
		sei();

		while(!usbcdc_can_putc(1)){
			usbcdc_poll();
		}
		usbcdc_putc(c);
	}
    }
    return 0;
}
