/*
 * isp.h - part of USBasp
 *
 * Autor..........: Thomas Fischl <tfischl@gmx.de>
 * Description....: Provides functions for communication/programming
 *                  over ISP interface
 * Licence........: GNU GPL v2 (see Readme.txt)
 * Creation Date..: 2005-02-23
 * Last change....: 2009-02-28
 */

#ifndef __isp_h_included__
#define	__isp_h_included__

#ifndef uchar
#define	uchar	unsigned char
#endif

#if 0
#define	ISP_OUT   PORTB
#define ISP_IN    PINB
#define ISP_DDR   DDRB
#define ISP_RST   PB0
#define ISP_MOSI  PB2
#define ISP_MISO  PB3
#define ISP_SCK   PB1
#endif

#define ISPCMD_ISP_CONF		    0xac
//  #define ISPCONF_ENABLE	    0x53
  #define ISPCONF_CHIP_ERASE	    0x80
//  #define ISPCONF_WRITE_FUSE_L	    0xa0
//  #define ISPCONF_WRITE_FUSE_H	    0xa8
//  #define ISPCONF_WRITE_FUSE_EX	    0xa4
//  #define ISPCONF_WRITE_LOCKBIT	    0xe0

  #define ISPCONF_RESET		    0x59 /* PDI */
  #define ISPCONF_SET_SPACE	    0x01 /* PDI */

#define ISPCMD_READ_SIGNATURE	    0x30
#define ISPCMD_READ_EEPROM	    0xa0
#define ISPCMD_WRITE_EEPROM	    0xc0
#define ISPCMD_CHK_BUSY	    	    0xf0

//#define ISPCMD_READ_FLASH_L	    0x20
//#define ISPCMD_READ_FLASH_H	    0x28

//#define ISPCMD_READ_FUSE_L	    0x50	/* lower/ext */
//#define ISPCMD_READ_FUSE_H	    0x58	/* upper/lockbit */

struct pgm_ops {
	/* polling method : optional */
	void  (* pgm_poll)(void);

	/* Close connection to target device */
	void  (* pgm_disconnect)(void);

	/* read and write 4 bytes from isp */
	void  (* pgm_transmit)(uchar *out, uchar *in);

	uchar (* pgm_enable)(void);

	/* read byte from eeprom at given address */
	uchar (* pgm_read_space0)(uint16_t address);

	/* write byte to eeprom at given address */
	uchar (* pgm_write_space0)(uint16_t address, uchar data);

	/* read byte from flash at given address */
	uchar (* pgm_read_space1)(uint32_t address);

	/* write byte to flash at given address */
	/* pollmode 1: page mode */
	uchar (* pgm_write_space1)(uint32_t address, uchar data, uchar pollmode);
	/* notify block size : optional */
	void (* pgm_blk_start)(uchar size, int16_t pgmsk);

	uchar (* pgm_open_page)(uint32_t address, uchar pollvalue);
	uchar (* pgm_close_page)(uint32_t address, uchar pollvalue);
};

/* Prepare connection to target device (AVR type through SPI) */
void ispConnect(struct pgm_ops *ops, uchar sck_opt, uchar chan);
void pditpiConnect(struct pgm_ops *ops, uchar sck_opt, uchar is_tpi);

#endif /* __isp_h_included__ */
